@@ -7,12 +7,14 @@ knitr::opts_chunk$set(
 
 # If any of the required packages are unavailable,
 # don't re-run the code
+# nolint start
 required <- c("dplyr", "ggplot2", "tidyr", "cmstatr")
 if (!all(unlist(lapply(required, function(pkg) {
     requireNamespace(pkg, quietly = TRUE)}
   )))) {
   knitr::opts_chunk$set(eval = FALSE)
 }
+# nolint end
 
 ## ----message=FALSE------------------------------------------------------------
 library(dplyr)
@@ -51,6 +53,18 @@ dat %>%
   theme_bw() +
   ggtitle("Batch Plot")
 
+## -----------------------------------------------------------------------------
+dat %>%
+  ggplot(aes(x = batch, y = strength)) +
+  geom_boxplot() +
+  geom_jitter_failure_mode(aes(color = failure_mode, shape = failure_mode), width = 0.25) +
+  geom_hline(aes(yintercept = value),
+             data = b_basis_pooled$basis %>% rename(condition = group),
+             color = "blue") +
+  facet_grid(. ~ condition) +
+  theme_bw() +
+  ggtitle("Batch Plot with Failure Modes")
+
 ## -----------------------------------------------------------------------------
 dat %>%
   ggplot(aes(x = strength, color = condition)) +
